unit mainform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OLEdbDef, ADOdb, DAOdb, DMOdb, Grids, ExtCtrls, StdCtrls, ComCtrls;

type
  TfrmMain = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Server: TEdit;
    Label2: TLabel;
    Database: TEdit;
    Label3: TLabel;
    UserID: TEdit;
    Label4: TLabel;
    Password: TEdit;
    btnConnect: TButton;
    Panel1: TPanel;
    Label5: TLabel;
    btnRun: TButton;
    StatusBar: TStatusBar;
    Query: TMemo;
    btnClose: TButton;
    rbDMO: TRadioButton;
    rbADO: TRadioButton;
    rbDAO: TRadioButton;
    Label6: TLabel;
    DaoPath: TEdit;
    btnExec: TButton;
    procedure btnConnectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnRunClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure rbDMOClick(Sender: TObject);
    procedure rbADOClick(Sender: TObject);
    procedure rbDAOClick(Sender: TObject);
    procedure btnExecClick(Sender: TObject);
  private
    db: TOLEDatabase;
    Q: TOLEQuery;
    rs: integer;
    procedure CheckControls;
    procedure DisplayResults;
  public
    procedure Status(const msg: string);
  end;

var
  frmMain: TfrmMain;

implementation

uses
  ResForm;

{$R *.DFM}

const
  MAXRS = 9;    // up to 10 result sets

var
  fResults: array[0..MAXRS] of TfrmResults;


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  StatusBar.Panels[0].Text := 'not connected';
  rs := 0;
end;


procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  db.Free;
end;


procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;


procedure TfrmMain.Panel1Resize(Sender: TObject);
begin
  Query.Height := Panel1.Height - 24;
end;


procedure TfrmMain.CheckControls;
begin
  Database.Enabled := Server.Enabled;
  UserID.Enabled := Server.Enabled;
  Password.Enabled := Server.Enabled;
  DaoPath.Enabled := not Server.Enabled;
end;

procedure TfrmMain.rbDMOClick(Sender: TObject);
begin
  Label1.Caption := 'Server';
  Server.Enabled := TRUE;
  CheckControls;
end;


procedure TfrmMain.rbADOClick(Sender: TObject);
begin
  Label1.Caption := 'ODBC DSN';
  Server.Enabled := TRUE;
  CheckControls;
end;


procedure TfrmMain.rbDAOClick(Sender: TObject);
begin
  Server.Enabled := FALSE;
  CheckControls;
end;


procedure TfrmMain.Status;
begin
  with StatusBar do
    begin
      Panels[0].Text := msg;
      Update;
    end;
end;


procedure TfrmMain.btnConnectClick(Sender: TObject);
var
  start: TDateTime;
begin
  Status('connecting');
  Screen.Cursor := crHourGlass;
  start := Now;
  try
    if (rbDMO.Checked) then
      db := TDMODatabase.Create(Server.Text, Database.Text, UserID.Text, Password.Text)
    else if (rbADO.Checked) then
      db := TADODatabase.Create(Server.Text, Database.Text, UserID.Text, Password.Text)
    else
      db := TDAODatabase.Create(DaoPath.Text);
    db.Connect;
  finally
    Screen.Cursor := crDefault;
  end;
  Status(Format('connected in %.1f s', [(Now-start) *24 *3600]));
  btnRun.Enabled := TRUE;
  btnConnect.Enabled := FALSE;
  btnExec.Enabled := TRUE;
end;


procedure TfrmMain.DisplayResults;
var
  cc, rr: integer;
begin
  Application.CreateForm(TfrmResults, fResults[rs]);
  with fResults[rs], Q do
    try
      Caption := Format('Result Set %d', [rs+1]);
      Answer.ColCount := FieldCount;
      for cc := 0 to FieldCount -1 do
        Answer.Cells[cc, 0] := FieldName(cc);
      rr := 0;
      while not EOF do
        begin
          Inc(rr);
          if (rr >= Answer.RowCount) then
            Answer.RowCount := rr +10;
          for cc := 0 to FieldCount -1 do
            Answer.Cells[cc, rr] := FieldByNumber(cc);
          Next;
        end;
      Show;
    finally
      Inc(rs);
    end;
end;


procedure TfrmMain.btnRunClick(Sender: TObject);
var
  start: TDateTime;
begin
  while (rs > 0) do
    begin
      Dec(rs);
      fResults[rs].Free;
    end;
  Status('executing query');
  Screen.Cursor := crHourglass;
  try
    start := Now;
    Q := db.CreateQueryObject;
    with Q do
      try
        SQL := Query.Text;
        Open;
        Status(Format('resultset available in %.1f s', [(Now -start) *24 *3600]));
        DisplayResults;
        while (rs < MAXRS) and (NextRecordSet) do
          DisplayResults;
      finally
        Free;
      end;
    Status(Format('completed in %.1f s', [(Now -start) *24 *3600]));
  finally
    Screen.Cursor := crdefault;
  end;
end;


procedure TfrmMain.btnExecClick(Sender: TObject);
var
  start: TDateTime;
begin
  while (rs > 0) do
    begin
      Dec(rs);
      fResults[rs].Free;
    end;
  Status('executing query');
  Screen.Cursor := crHourglass;
  try
    start := Now;
    Q := db.CreateQueryObject;
    with Q do
      try
        SQL := Query.Text;
        ExecSQL;
      finally
        Free;
      end;
    Status(Format('completed in %.1f s', [(Now -start) *24 *3600]));
  finally
    Screen.Cursor := crdefault;
  end;
end;

end.
